;$Id: shapelets_make_chi2_grid.pro, v1$
;
; Copyright  2005 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.astro.caltech.edu/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public Licence as published
; by the Free Software Foundation; either version 2 of the Licence, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public Licence for more details.
;
; You should have received a copy of the GNU General Public Licence
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
;
;+
; NAME:
;      SHAPELETS_MAKE_CHI2_GRID
;
; CATEGORY:
;      Shapelets.
;
; PURPOSE:
;      Compute the chi2 difference between the input and the reconstructed
;      image on a grid of values of beta and n_max.
;
; INPUTS:
;      PSTAMP - object structure from shapelets_extract_pstamp.pro
;
; OPTIONAL INPUTS:
;      PLOTIT: 1: plot chi2 array; 2: also plot reconstructed images
;                 beta_range,n_ran: beta and nmax range (default: educated guess)
;      N_BETA: number of beta values to consider (default=10)
;      silent: print results
;      X0: centroid position (default: guess it from image moments)
;
; KEYWORD PARAMETERS:
;      None.
;
; OUTPUTS:
;      chi2_grid: result structure
;
; MODIFICATION HISTORY:
;      Jul 05 - Updated by Richard Massey.
;      Apr 02 - Modified by AR to use least-square fitting rather than
;               linear decomposition
;      Aug 00 - Written by A. Refregier
;-

function shapelets_make_chi2_grid, PSTAMP,                  $
                                   N_MAX_RANGE=n_max_range, $
			           BETA_RANGE=beta_range,   $
			           FOCUS=focus,             $
				   N_BETA=n_beta,	    $
			           X0=x0,		    $
			           PSF=psf,		    $
			           SKY=sky,		    $
                                   PLOTIT=plotit,	    $
			           SILENT=silent

; Initial declarations
;sf=size(pstamp.ima)
;n_f=[sf[1],sf[2]]
n_d=pstamp.n_pixels[0]*pstamp.n_pixels[1]
if not keyword_set(n_beta) then n_beta=10      ; number of beta values

; compute guess for beta and n_max
shapelets_guess_nmaxbeta, pstamp, guess, th_min_geom=0.2
help,guess,/st

;shapelets_image_moments,pstamp.image,mom             ; compute first few moments
mom=shapelets_image_moments(pstamp.image)
th_max=mom.r*1.
th_min=1.
beta_guess=sqrt(th_max*th_min)
n_guess=round(th_max/th_min)
if not keyword_set(x0) then x0=mom.xc
print,'initial guess: w, n,x0:',beta_guess,n_guess,x0

; Select range of beta and n_max 
if not keyword_set(beta_range) then beta_range=[beta_guess/2.,beta_guess*2.]
if not keyword_set(n_beta) then n_beta=10
beta=xgen(beta_range[0],beta_range[1],np=n_beta)
if not keyword_set(n_max_range) then n_max_range=[0,n_guess*2<20]  ; range of n_max values
n_n_max=n_max_range[1]-n_max_range[0]+1
n_max=indgen(n_n_max)+n_max_range[0]
n_coeffs=(1+n_max)*(2+n_max)/2   ; number of coefficients for given n_max

; decompose the image for different values of beta and max and compute chi^2
!p.region=0
if keyword_set(plotit) then begin
  if plotit eq 2 then begin
    !p.multi=[0,3,3]
  endif
endif
;shapelets_plot_image,f
chi2_grid=fltarr(n_n_max,n_beta)
chi2_min=!values.f_infinity
if keyword_set(plotit) then r=1
for i=0,n_beta-1 do begin
  for j=0,n_n_max-1 do begin
    message,"Trying n_max="+strtrim(n_max[j],2)+", beta="+strtrim(beta[i],2),/info,noprint=silent
    shapelets_decomp,pstamp.image,beta[i],n_max[j],decomp,recomp,x0=x0,/over,/ls,$
      noise=pstamp.noise,psf=psf,sky=sky
    chi2_grid[j,i]=decomp.chisq[1]
    if keyword_set(plotit) then begin
      if plotit eq 2 then begin 
        if ((i*n_n+j) mod 9 eq 0) then read,test
        print,'beta, n, chi^2:',beta[i], n_max[j],chi2a[j,i]
        shapelets_plot_image,r
      endif
    endif
    if chi2_grid[j,i] lt chi2_min then begin
      chi2_min=chi2_grid[j,i]
      beta_best_fit=beta[i]
      n_max_best_fit=n_max[j] 
    endif
  endfor
endfor

; Report best fit values
message,"best fit: beta, n_max: "+string(beta_best_fit)+string(n_max_best_fit),/info,noprint=silent
message,"theta_min,theta_max:"+string(beta_best_fit/sqrt(n_max_best_fit+1))+string(beta_best_fit*sqrt(n_max_best_fit+1)),/info,noprint=silent

; Store best fit values in structure
chi2_grid={name:pstamp.name,              $
           type:"chisq_grid",             $
	   beta_best_fit:beta_best_fit,   $
           n_max_best_fit:n_max_best_fit, $
	   beta:beta,                     $
	   n_max:n_max,                   $
	   chi2_grid:chi2_grid,           $
	   beta_guess:beta_guess,         $
	   nmax_guess:n_guess}

; 
if arg_present(focus) then begin
  
endif

return,chi2_grid

end


