pro shapelets_read_image, IMAGE, FILENAME,               $
                          FULL_PATH=full_path,           $
                          ESTIMATE_NOISE=estimate_noise, $
                          NOISE_LEVEL=noise_level,       $
                          N_GROW=n_grow,                 $
			  SKY_SUBTRACT=sky_subtract,     $
			  SILENT=silent

;$Id: shapelets_read_image.pro, v2$
;
; Copyright  2005 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.ast.cam.ac.uk/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public Licence as published
; by the Free Software Foundation; either version 2 of the Licence, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public Licence for more details.
;
; You should have received a copy of the GNU General Public Licence
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
;
;+
; NAME:
;         SHAPELETS_READ_IMAGE
;
; PURPOSE:
;         Read in fits image, plus segmentation and inverse variance pixel
;         map if available (it looks for the same filename, but with
;         _objects and _weight added before the extension).
;
; CATEGORY:
;         Data I/O.
;
; INPUTS:
;         FILENAME    - Name of .fits image file.
;
; OPTIONAL INPUTS:
;         NOISE_LEVEL - An estimate of the rms background noise.
;         N_GROW      - Number of times SExtractor segmentation map is "grown"
;                       to mask objects during noise estimation
;
; KEYWORD PARAMETERS:
;         FULL_PATH   - If this keyword is set, the routine expects "filename"
;                       to contain the full path and extension of the image file.
;                       Without it, the default is to look in the directory
;                       specified within shapelets_paths.pro.
;         ESTIMATE_NO - If this is just set to 1, estimates the global noise 
;                       level from the entire image.
;         SILENT      - Operate silently.
;
; OUTPUTS:
;         Returns an image structure, containing image, header, noise map, etc.
;
; EXAMPLE USE:
;         shapelets_read_image,image,"example"
;
; MODIFICATION HISTORY:
;         Jul 05 - SExtractor's negative segmentation map fixed by Matt Ferry.
;         Jul 05 - Global noise estimation improved by RM.
;         May 05 - FULLPATH keyword changed to FULL_PATH by RM.
;         Apr 05 - Format of noise tag in image structure changed by RM.
;         Mar 05 - Mask file added by RM.
;         Jul 04 - Segmentation map file names changed from _objects.fits to
;                  _seg.fits by AR.
;         May 04 - Weight maps changed from _noise.fits to _weight.fits by RM.
;         Mar 04 - Modified by AR to clarify noise map.
;         Apr 02 - Generalised to accept different filenames by Richard Massey.
;         Mar 02 - Written by Alexandre Refregier (adapted from RM's shex.pro).
;-

;
; Decide file name to look for
;
if keyword_set(filename) then begin
  file=filename ; Don't disrupt input variables
endif else message,"You must specify an image file name!"
if keyword_set(full_path) then begin
  ; Strip file extension
  if strpos(strlowcase(file),".fit") ne -1 then $
    file=strmid(file,0,strpos(strlowcase(file),'.'))
endif else begin
  ; Add relevant path name
  file=shapelets_paths(3,silent=silent)+file
endelse
if file_test(file+'.fits') then begin
  message,'Reading in image data from '+file+'.fits',/info,noprint=silent
endif else message,"Cannot find image "+file+".fits!"


;
; Read fits image
;
fits_read,file+'.fits',image,header


;
; Determine key image properties
;
n_pixels=size(image,/dimensions)
pixel_scale_line=where(strmid(header[*],0,9) eq "PIXSIZE =")
if pixel_scale_line[0] eq -1 then pixel_scale=1. $
else pixel_scale=float(strmid(header[pixel_scale_line],9,22))
photo_zp=0.


;
; Subtract constant sky background level
;
if keyword_set(sky_subtract) then begin
  ;sky_background=2.5*median(image.image)-1.5*mean(image.image)
  highbad=max(image)
  sky,image,sky_background,highbad=highbad,/SILENT
  message,"Subtracting a sky background level of "+strtrim(sky_background,2),/info,noprint=silent
  image-=sky_background
endif



;
; Store in image-type IDL structure
;
image={name:filename,           $
       type:"image",            $
       history:strarr(1),       $
       image:image,             $
       n_pixels:n_pixels,       $
       pixel_scale:pixel_scale, $
       photo_zp:photo_zp,       $
       header:header}


;
; Read in or construct image mask
;
if file_test(file+'_mask.fits') then begin
	message,'Reading in image mask',/info,noprint=silent
	fits_read,file+'_mask.fits',mask
     if size(mask,/TYPE) ne 1 then mask=byte(temporary(mask))
endif else begin
     message,'No mask available',/info,noprint=silent
     mask=0B
endelse
image=create_struct(temporary(image),"mask",mask)
delvarx,mask


;
; Read in "segmented image" created by SExtractor
; (this flags every pixel with the SEx ID of the object it belongs to -
;  or zero, if it is part of the background.)
;
if file_test(file+'_seg.fits') then begin
  found_seg_map=1B
  message,'Reading in segmented pixel map',/info,noprint=silent
  fits_read,file+'_seg.fits',seg
  ; If too many objects, SExtractor's integer system wraps around, jumping
  ; from positive to negative. This quick fix is needed if there are more
  ; 2^15 objects, and works as long as there are fewer than 2^16.
  if min(seg) lt 0  then begin
    message,'Correcting segmentation map for negative indices...',/info,noprint=silent
    seg+=(seg lt 0)*65536L
  endif
endif else begin
  found_seg_map=0B
  message,'Segmented SExtractor pixel map does not exist!',/info,$
        noprint=silent
  seg=0B
endelse
image=create_struct(temporary(image),"seg",seg)
delvarx,seg


;
; Read in or construct noise map
;
if keyword_set(noise_level) then begin
  message,'Storing input noise level',/info,noprint=silent
  noise=1./((noise_level[0]^2)>1e-10)
endif else if keyword_set(estimate_noise) then begin
  ;message,'Manufacturing artificial noise data',/info,noprint=silent
  ;background=2.5*median(image.image)-1.5*mean(image.image)
  ;noise=1./(mean(abs(image.image-background)))^2
  if found_seg_map then begin
    objects=abs(image.seg)<1
    roi=where(objects,n_roi)
    if n_roi gt 0 then begin
      if not keyword_set(n_grow) then n_grow=2
      grown_objects=-objects
      for k=-n_grow,n_grow do grown_objects=+shift(objects,k)
      tr_objects=transpose(objects)
      for k=-n_grow,n_grow do grown_objects+=transpose(shift(tr_objects,k))
      for k=1,(n_grow+1)/2 do begin
   	for i=-1,1,2 do begin
 	  for j=-1,1,2 do begin
   	    grown_objects+=transpose(shift(transpose(shift(objects,i*k)),j*k))
   	  endfor
   	endfor
      endfor
      objects=grown_objects<1
      roi=where(objects,n_roi)
      delvarx,objects
      temp_image=image.image
      if n_roi gt 0 then begin
        highbad=max(temp_image)+1
        temp_image[roi]=highbad
      endif
      delvarx,roi
      sky,temp_image,mean_background,noise,highbad=highbad,/SILENT
      delvarx,temp_image
    endif else sky,image.image,mean_background,noise,/SILENT
  endif else sky,image.image,mean_background,noise,/SILENT
  noise=1./noise^2
endif else if file_test(file+'_weight.fits') then begin
  message,'Reading in inverse variance (weight) map',/info,noprint=silent
  fits_read,file+'_weight.fits',noise
endif else begin
  message,'No inverse variance (weight) map available',/info,noprint=silent
  noise=0.
endelse
image=create_struct(temporary(image),"noise",noise)
delvarx,noise


;
; Write our success story
;
message,'Image '+file+'.fits now in memory',/info,noprint=silent

end
